import java.io.*; 
import java.util.Scanner;
import java.util.ArrayList;

/******************************************************************************
*  Generates random monsters, as for wandering encounters (singleton pattern).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-07-18
*  @version  1.0
******************************************************************************/

public class MonsterRandomizer {

	//--------------------------------------------------------------------------
	//  Constants
	//--------------------------------------------------------------------------

	/** Name of file with monster information. */
	final String MONSTER_LEVEL_MATRIX_FILE = "MonsterLevelMatrix.csv";

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** The singleton class instance. */
	static MonsterRandomizer instance = null;

	/** Matrix of what level monster appears. */
	ArrayList<ArrayList<Integer>> monsterLevelMatrix;

	/** Tables of monsters at each level. */
	ArrayList<ArrayList<Monster>> monsterLevelTables;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

 	/**
 	*  Constructor (read from dedicated file).
 	*/
 	protected MonsterRandomizer () throws IOException {
		readMonsterLevelMatrix();
		makeMonsterLevelTables();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Access the singleton class instance.
	*/
	public static MonsterRandomizer getInstance() {
		if (instance == null) {
			try {
				instance = new MonsterRandomizer();
			}
			catch (IOException e) {
				System.out.println("Failed to read the Monster Level Matrix file.");
			}
		}
		return instance;
	}

	/**
	*  Read in the monster level matrix.
	*/
	void readMonsterLevelMatrix () throws IOException {
		String[][] table = CSVReader.readFile(MONSTER_LEVEL_MATRIX_FILE);
		monsterLevelMatrix = new ArrayList<ArrayList<Integer>>();
		for (int i = 1; i < table.length; i++) {
			ArrayList<Integer> thisRow = new ArrayList<Integer>();
			for (int j = 0; j < table[i].length; j++) {
				thisRow.add(CSVReader.parseInt(table[i][j]));
			}
			monsterLevelMatrix.add(thisRow);
		}
	}

	/**
	*  Compile the table of monsters at each level.
	*/
	void makeMonsterLevelTables () {

		// Initialize the tables
		int maxLevel = monsterLevelMatrix.get(0).size() - 1;
		monsterLevelTables = new ArrayList<ArrayList<Monster>>(maxLevel);
		for (int i = 0; i < maxLevel; i++) {
			 monsterLevelTables.add(new ArrayList<Monster>());		
		}		
		
		// Sort monster list into these tables
		MonsterList monList = MonsterList.getInstance();
		for (int i = 0; i < monList.size(); i++) {
			Monster monster = monList.get(i);
			int tableLevel = monster.getTableLevel();
			if (1 <= tableLevel && tableLevel <= maxLevel) {
				monsterLevelTables.get(tableLevel - 1).add(monster);
			}		
		}
	}

	/**
	*  Roll a new random monster on a given dungeon level.
	*/
	public Monster randomMonsterOnDungeonLevel (int dungeonLevel) {
		int tableLevel = randomTableLevel(dungeonLevel);
		return randomMonsterOnTableLevel(tableLevel);
	}

	/**
	*  Roll a random table level for a given dungeon level.
	*/
	int randomTableLevel (int dungeonLevel) {
		ArrayList<Integer> row = getMatrixRow(dungeonLevel);
		int roll = new Dice(6).roll();
		for (int i = row.size() - 1; i > 0; i--) {
			int val = row.get(i);
			if (val != 0 && roll >= val) return i;
		}
		return -1;
	}

	/**
	*  Roll a new random monster of a given table level.
	*/
	public Monster randomMonsterOnTableLevel (int tableLevel) {
		ArrayList<Monster> levelArray 
			= monsterLevelTables.get(tableLevel - 1);		
		int idx = Dice.roll(levelArray.size()) - 1;
		return levelArray.get(idx).spawn();
	}

	/**
	*  Find the correct row of the matrix for a given level.
	*/
	ArrayList<Integer> getMatrixRow (int dungeonLevel) {
		for (int i = monsterLevelMatrix.size() - 1; i > -1; i--) {
			ArrayList<Integer> row = monsterLevelMatrix.get(i);
			if (dungeonLevel >= row.get(0)) return row;
		}
		return null;
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		MonsterRandomizer mr = MonsterRandomizer.getInstance();
	
		// Five random monsters from 1st-level table
		System.out.println("Monsters from 1st-level table:");
		for (int i = 0; i < 5; i++) {
			System.out.println(mr.randomMonsterOnTableLevel(1));
		}
		System.out.println();

		// Five random monsters from 4th-level table
		System.out.println("Monsters from 4th-level table:");
		for (int i = 0; i < 5; i++) {
			System.out.println(mr.randomMonsterOnTableLevel(4));
		}
		System.out.println();

		// Five random monsters from 5th-level table
		System.out.println("Monsters from 5th-level table:");
		for (int i = 0; i < 5; i++) {
			System.out.println(mr.randomMonsterOnTableLevel(5));
		}
		System.out.println();
		
		// Five random monsters on dungeon level 7
		System.out.println("Monsters from dungeon level 7:");
		for (int i = 0; i < 5; i++) {
			System.out.println(mr.randomMonsterOnDungeonLevel(7));
		}
		System.out.println();
	
		// Five random monsters on dungeon level 20
		System.out.println("Monsters from dungeon level 20:");
		for (int i = 0; i < 5; i++) {
			System.out.println(mr.randomMonsterOnDungeonLevel(20));
		}
		System.out.println();
	}
}

